==> falcon_caching-1.0.1-py3-none-any.whl: falcon_caching/async_backends/redis.py <==
# -*- coding: utf-8 -*-
"""
    falcon_caching.async_backends.redis
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    The redis caching backend (async).

    :copyright: (c) 2022 by Zoltan Fedor.
    :license: MIT, see LICENSE for more details.

    :copyright: (c) 2018 by Peter Justin.
    :copyright: (c) 2010 by Thadeus Burgess.
    :license: BSD, see LICENSE for more details.
"""
from falcon_caching.async_backends.base import BaseCache, iteritems_wrapper

try:
    import cPickle as pickle
except ImportError:  # pragma: no cover
    import pickle


class Redis(BaseCache):
    """Uses the Redis key-value store as a cache backend.

    The first argument can be either a string denoting address of the Redis
    server or an object resembling an instance of a redis.Redis class.

    Note: Python Redis API already takes care of encoding unicode strings on
    the fly.

    :param host: address of the Redis server or an object which API is
                 compatible with the official Python Redis client (redis-py).
    :param port: port number on which Redis server listens for connections.
    :param password: password authentication for the Redis server.
    :param db: db (zero-based numeric index) on Redis Server to connect.
    :param default_timeout: the default timeout that is used if no timeout is
                            specified on :meth:`~BaseCache.set`. A timeout of
                            0 indicates that the cache never expires.
    :param key_prefix: A prefix that should be added to all keys.

    Any additional keyword arguments will be passed to ``redis.Redis``.
    """

    def __init__(
        self,
        host="localhost",
        port=6379,
        password=None,
        db=0,
        default_timeout=300,
        key_prefix=None,
        **kwargs
    ):
        super(Redis, self).__init__(default_timeout)
        if host is None:
            raise ValueError("Redis host parameter may not be None")
        if isinstance(host, str):
            try:
                import coredis
            except ImportError:
                raise RuntimeError("no coredis module found")
            if kwargs.get("decode_responses", None):
                raise ValueError(
                    "decode_responses is not supported by " "Redis."
                )
            client = coredis.StrictRedis(
                host=host, port=port, password=password, db=db, **kwargs
            )
        else:
            client = host

        self._write_client = self._read_clients = client
        self.key_prefix = key_prefix or ""

    def _get_prefix(self):
        return (
            self.key_prefix
            if isinstance(self.key_prefix, str)
            else self.key_prefix()
        )

    def _normalize_timeout(self, timeout):
        timeout = BaseCache._normalize_timeout(self, timeout)
        if timeout == 0:
            timeout = -1
        return timeout

    def dump_object(self, value):
        """Dumps an object into a string for redis.  By default it serializes
        integers as regular string and pickle dumps everything else.
        """
        t = type(value)
        if t == int:
            return str(value).encode("ascii")
        return b"!" + pickle.dumps(value)

    def load_object(self, value):
        """The reversal of :meth:`dump_object`.  This might be called with
        None.
        """
        if value is None:
            return None
        if value.startswith(b"!"):
            try:
                return pickle.loads(value[1:])
            except pickle.PickleError:
                return None
        try:
            return int(value)
        except ValueError:
            # before 0.8 we did not have serialization.  Still support that.
            return value

    async def get(self, key):
        return self.load_object(
            await self._read_clients.get(self._get_prefix() + key)
        )

    async def get_many(self, *keys):
        if self.key_prefix:
            keys = [self._get_prefix() + key for key in keys]
        return [self.load_object(x) for x in await self._read_clients.mget(keys)]

    async def set(self, key, value, timeout=None):
        timeout = self._normalize_timeout(timeout)
        dump = self.dump_object(value)
        if timeout == -1:
            result = await self._write_client.set(
                key=self._get_prefix() + key, value=dump
            )
        else:
            result = await self._write_client.setex(
                key=self._get_prefix() + key, value=dump, seconds=timeout
            )
        return result

    async def add(self, key, value, timeout=None):
        timeout = self._normalize_timeout(timeout)
        dump = self.dump_object(value)
        return await self._write_client.setnx(
            key=self._get_prefix() + key, value=dump
        ) and self._write_client.expire(
            key=self._get_prefix() + key, seconds=timeout
        )

    async def set_many(self, mapping, timeout=None):
        timeout = self._normalize_timeout(timeout)
        # Use transaction=False to batch without calling redis MULTI
        # which is not supported by twemproxy
        pipe = await self._write_client.pipeline(transaction=False)

        for key, value in iteritems_wrapper(mapping):
            dump = self.dump_object(value)
            if timeout == -1:
                await pipe.set(key=self._get_prefix() + key, value=dump)
            else:
                await pipe.setex(
                    key=self._get_prefix() + key, value=dump, seconds=timeout
                )
        return await pipe.execute()

    async def delete(self, key):
        return await self._write_client.delete(self._get_prefix() + key)

    async def delete_many(self, *keys):
        if not keys:
            return
        if self.key_prefix:
            keys = [self._get_prefix() + key for key in keys]
        return await self._write_client.delete(*keys)

    async def has(self, key):
        return await self._read_clients.exists(self._get_prefix() + key)

    async def clear(self):
        status = False
        if self.key_prefix:
            keys = await self._read_clients.keys(self._get_prefix() + "*")
            if keys:
                status = await self._write_client.delete(*keys)
        else:
            status = await self._write_client.flushdb(asynchronous=True)
        return status

    async def inc(self, key, delta=1):
        return await self._write_client.incr(
            key=self._get_prefix() + key, amount=delta
        )

    async def dec(self, key, delta=1):
        return await self._write_client.decr(
            key=self._get_prefix() + key, amount=delta
        )

    async def unlink(self, *keys):
        """when redis-py >= 3.0.0 and redis > 4, support this operation
        """
        if not keys:
            return
        if self.key_prefix:
            keys = [self.key_prefix + key for key in keys]

        unlink = getattr(self._write_client, "unlink", None)
        if unlink is not None and callable(unlink):
            return await  self._write_client.unlink(*keys)
        return await self._write_client.delete(*keys)


class RedisSentinel(Redis):
    """Uses the Redis key-value store as a cache backend.

    The first argument can be either a string denoting address of the Redis
    server or an object resembling an instance of a redis.Redis class.

    Note: Python Redis API already takes care of encoding unicode strings on
    the fly.


    :param sentinels: A list or a tuple of Redis sentinel addresses.
    :param master: The name of the master server in a sentinel configuration.
    :param password: password authentication for the Redis server.
    :param db: db (zero-based numeric index) on Redis Server to connect.
    :param default_timeout: the default timeout that is used if no timeout is
                            specified on :meth:`~BaseCache.set`. A timeout of
                            0 indicates that the cache never expires.
    :param key_prefix: A prefix that should be added to all keys.

    Any additional keyword arguments will be passed to
    ``redis.sentinel.Sentinel``.
    """

    def __init__(
        self,
        sentinels=None,
        master=None,
        password=None,
        db=0,
        default_timeout=300,
        key_prefix=None,
        **kwargs
    ):
        super(RedisSentinel, self).__init__(default_timeout)

        try:
            import coredis.sentinel
        except ImportError:
            raise RuntimeError("no coredis module found")

        if kwargs.get("decode_responses", None):
            raise ValueError(
                "decode_responses is not supported by " "Redis."
            )

        sentinels = sentinels or [("127.0.0.1", 26379)]
        sentinel_kwargs = {
            key[9:]: value
            for key, value in kwargs.items()
            if key.startswith("sentinel_")
        }
        kwargs = {
            key[9:]: value
            for key, value in kwargs.items()
            if not key.startswith("sentinel_")
        }

        sentinel = coredis.sentinel.Sentinel(
            sentinels=sentinels,
            password=password,
            db=db,
            sentinel_kwargs=sentinel_kwargs,
            **kwargs
        )

        self._write_client = sentinel.master_for(master)
        self._read_clients = sentinel.slave_for(master)

        self.key_prefix = key_prefix or ""

==> falcon_caching-1.1.0-py3-none-any.whl: falcon_caching/async_backends/redis.py <==
# -*- coding: utf-8 -*-
"""
    falcon_caching.async_backends.redis
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    The redis caching backend (async).

    :copyright: (c) 2022 by Zoltan Fedor.
    :license: MIT, see LICENSE for more details.

    :copyright: (c) 2018 by Peter Justin.
    :copyright: (c) 2010 by Thadeus Burgess.
    :license: BSD, see LICENSE for more details.
"""
from falcon_caching.async_backends.base import BaseCache, iteritems_wrapper

try:
    import cPickle as pickle
except ImportError:  # pragma: no cover
    import pickle


class Redis(BaseCache):
    """Uses the Redis key-value store as a cache backend.

    The first argument can be either a string denoting address of the Redis
    server or an object resembling an instance of a redis.Redis class.

    Note: Python Redis API already takes care of encoding unicode strings on
    the fly.

    :param host: address of the Redis server or an object which API is
                 compatible with the official Python Redis client (redis-py).
    :param port: port number on which Redis server listens for connections.
    :param password: password authentication for the Redis server.
    :param db: db (zero-based numeric index) on Redis Server to connect.
    :param default_timeout: the default timeout that is used if no timeout is
                            specified on :meth:`~BaseCache.set`. A timeout of
                            0 indicates that the cache never expires.
    :param key_prefix: A prefix that should be added to all keys.

    Any additional keyword arguments will be passed to ``redis.Redis``.
    """

    def __init__(
        self,
        host="localhost",
        port=6379,
        password=None,
        db=0,
        default_timeout=300,
        key_prefix=None,
        **kwargs
    ):
        super(Redis, self).__init__(default_timeout)
        if host is None:
            raise ValueError("Redis host parameter may not be None")
        if isinstance(host, str):
            try:
                import coredis
            except ImportError:
                raise RuntimeError("no coredis module found")
            client = coredis.Redis(
                host=host, port=port, password=password, db=db, **kwargs
            )
        else:
            client = host

        self._write_client = self._read_clients = client
        self.key_prefix = key_prefix or ""

    def _get_prefix(self):
        return (
            self.key_prefix
            if isinstance(self.key_prefix, str)
            else self.key_prefix()
        )

    def _normalize_timeout(self, timeout):
        timeout = BaseCache._normalize_timeout(self, timeout)
        if timeout == 0:
            timeout = -1
        return timeout

    def dump_object(self, value):
        """Dumps an object into a string for redis.  By default it serializes
        integers as regular string and pickle dumps everything else.
        """
        t = type(value)
        if t == int:
            return str(value).encode("ascii")
        return b"!" + pickle.dumps(value)

    def load_object(self, value):
        """The reversal of :meth:`dump_object`.  This might be called with
        None.
        """
        if value is None:
            return None
        if value.startswith(b"!"):
            try:
                return pickle.loads(value[1:])
            except pickle.PickleError:
                return None
        try:
            return int(value)
        except ValueError:
            # before 0.8 we did not have serialization.  Still support that.
            return value

    async def get(self, key):
        return self.load_object(
            await self._read_clients.get(self._get_prefix() + key)
        )

    async def get_many(self, *keys):
        if self.key_prefix:
            keys = [self._get_prefix() + key for key in keys]
        return [self.load_object(x) for x in await self._read_clients.mget(keys)]

    async def set(self, key, value, timeout=None):
        timeout = self._normalize_timeout(timeout)
        dump = self.dump_object(value)
        if timeout == -1:
            result = await self._write_client.set(
                key=self._get_prefix() + key, value=dump
            )
        else:
            result = await self._write_client.setex(
                key=self._get_prefix() + key, value=dump, seconds=timeout
            )
        return result

    async def add(self, key, value, timeout=None):
        timeout = self._normalize_timeout(timeout)
        dump = self.dump_object(value)
        return await self._write_client.setnx(
            key=self._get_prefix() + key, value=dump
        ) and self._write_client.expire(
            key=self._get_prefix() + key, seconds=timeout
        )

    async def set_many(self, mapping, timeout=None):
        timeout = self._normalize_timeout(timeout)
        # Use transaction=False to batch without calling redis MULTI
        # which is not supported by twemproxy
        pipe = await self._write_client.pipeline(transaction=False)

        for key, value in iteritems_wrapper(mapping):
            dump = self.dump_object(value)
            if timeout == -1:
                await pipe.set(key=self._get_prefix() + key, value=dump)
            else:
                await pipe.setex(
                    key=self._get_prefix() + key, value=dump, seconds=timeout
                )
        return await pipe.execute()

    async def delete(self, key):
        return await self._write_client.delete([self._get_prefix() + key])

    async def delete_many(self, *keys):
        if not keys:
            return
        if self.key_prefix:
            keys = [self._get_prefix() + key for key in keys]
        return await self._write_client.delete(keys)

    async def has(self, key):
        return await self._read_clients.exists([self._get_prefix() + key])

    async def clear(self):
        status = False
        if self.key_prefix:
            keys = await self._read_clients.keys(self._get_prefix() + "*")
            if keys:
                status = await self._write_client.delete(keys)
        else:
            status = await self._write_client.flushdb(b"ASYNC")
        return status

    async def inc(self, key, delta=1):
        return await self._write_client.incrby(
            key=self._get_prefix() + key, increment=delta
        )

    async def dec(self, key, delta=1):
        return await self._write_client.decrby(
            key=self._get_prefix() + key, decrement=delta
        )

    async def unlink(self, *keys):
        """when redis-py >= 3.0.0 and redis > 4, support this operation
        """
        if not keys:
            return
        if self.key_prefix:
            keys = [self.key_prefix + key for key in keys]

        unlink = getattr(self._write_client, "unlink", None)
        if unlink is not None and callable(unlink):
            return await  self._write_client.unlink(keys)
        return await self._write_client.delete(keys)


class RedisSentinel(Redis):
    """Uses the Redis key-value store as a cache backend.

    The first argument can be either a string denoting address of the Redis
    server or an object resembling an instance of a redis.Redis class.

    Note: Python Redis API already takes care of encoding unicode strings on
    the fly.


    :param sentinels: A list or a tuple of Redis sentinel addresses.
    :param master: The name of the master server in a sentinel configuration.
    :param password: password authentication for the Redis server.
    :param db: db (zero-based numeric index) on Redis Server to connect.
    :param default_timeout: the default timeout that is used if no timeout is
                            specified on :meth:`~BaseCache.set`. A timeout of
                            0 indicates that the cache never expires.
    :param key_prefix: A prefix that should be added to all keys.

    Any additional keyword arguments will be passed to
    ``redis.sentinel.Sentinel``.
    """

    def __init__(
        self,
        sentinels=None,
        master=None,
        password=None,
        db=0,
        default_timeout=300,
        key_prefix=None,
        **kwargs
    ):
        super(RedisSentinel, self).__init__(default_timeout)

        try:
            import coredis.sentinel
        except ImportError:
            raise RuntimeError("no coredis module found")

        if kwargs.get("decode_responses", None):
            raise ValueError(
                "decode_responses is not supported by " "Redis."
            )

        sentinels = sentinels or [("127.0.0.1", 26379)]
        sentinel_kwargs = {
            key[9:]: value
            for key, value in kwargs.items()
            if key.startswith("sentinel_")
        }
        kwargs = {
            key[9:]: value
            for key, value in kwargs.items()
            if not key.startswith("sentinel_")
        }

        sentinel = coredis.sentinel.Sentinel(
            sentinels=sentinels,
            password=password,
            db=db,
            sentinel_kwargs=sentinel_kwargs,
            **kwargs
        )

        self._write_client = sentinel.master_for(master)
        self._read_clients = sentinel.slave_for(master)

        self.key_prefix = key_prefix or ""
